import express from 'express';

import MaterialValidate from './validate/material';
import MaterialModel from '../models/material'
import SampleModel from '../models/sample';
import IdValidate from './validate/id';
import res400 from './validate/res400';
import mongoose from 'mongoose';


const router = express.Router();

router.get('/materials', (req, res, next) => {
  if (!req.auth(res, ['read', 'write', 'maintain', 'dev', 'admin'], 'all')) return;

  MaterialModel.find({}).lean().exec((err, data) => {
    if (err) return next(err);
    res.json(data.map(e => MaterialValidate.output(e)).filter(e => e !== null));  // validate all and filter null values from validation errors
  });
});

router.get('/material/' + IdValidate.parameter(), (req, res, next) => {
  if (!req.auth(res, ['read', 'write', 'maintain', 'dev', 'admin'], 'all')) return;

  MaterialModel.findById(req.params.id).lean().exec((err, data) => {
    if (err) return next(err);
    if (data) {
      res.json(MaterialValidate.output(data));
    }
    else {
      res.status(404).json({status: 'Not found'});
    }
  });
});

router.put('/material/' + IdValidate.parameter(), (req, res, next) => {
  if (!req.auth(res, ['write', 'maintain', 'dev', 'admin'], 'basic')) return;

  const {error, value: material} = MaterialValidate.input(req.body, 'change');
  if (error) return res400(error, res);

  if (material.hasOwnProperty('name')) {
    MaterialModel.find({name: material.name}).lean().exec((err, data) => {
      if (err) return next(err);
      if (data.length > 0 && data[0]._id != req.params.id) {
        res.status(400).json({status: 'Material name already taken'});
        return;
      }
      else {
        f();
      }
    });
  }
  else {
    f();
  }

  function f() {  // to resolve async
    MaterialModel.findByIdAndUpdate(req.params.id, material, {new: true}).lean().exec((err, data) => {
      if (err) return next(err);
      if (data) {
        res.json(MaterialValidate.output(data));
      }
      else {
        res.status(404).json({status: 'Not found'});
      }
    });
  }
});

router.delete('/material/' + IdValidate.parameter(), (req, res, next) => {
  if (!req.auth(res, ['write', 'maintain', 'dev', 'admin'], 'basic')) return;

  // check if there are still samples referencing this material
  SampleModel.find({'material_id': new mongoose.Types.ObjectId(req.params.id)}).lean().exec((err, data) => {
    if (err) return next(err);
    if (data.length) {
      return res.status(400).json({status: 'Material still in use'});
    }
    MaterialModel.findByIdAndDelete(req.params.id).lean().exec((err, data) => {
      if (err) return next(err);
      if (data) {
        res.json({status: 'OK'})
      }
      else {
        res.status(404).json({status: 'Not found'});
      }
    });
  });
});

router.post('/material/new', (req, res, next) => {
  if (!req.auth(res, ['write', 'maintain', 'dev', 'admin'], 'basic')) return;

  // validate input
  const {error, value: material} = MaterialValidate.input(req.body, 'new');
  if (error) return res400(error, res);

  MaterialModel.find({name: material.name}).lean().exec((err, data) => {
    if (err) return next(err);
    if (data.length > 0) {
      res.status(400).json({status: 'Material name already taken'});
      return;
    }

    new MaterialModel(material).save((err, data) => {
      if (err) return next(err);
      res.json(MaterialValidate.output(data.toObject()));
    });
  });
});


module.exports = router;